#include <carmen/carmen.h>

static char *home = NULL;

static void
usage(int argc __attribute__ ((unused)), char *argv[]) 
{
  fprintf(stderr, "Usage: %s [OPTIONS] [MODES]\n", argv[0]);
  fprintf(stderr, "Options:\n");
  fprintf(stderr, "\t[--version]\n");
  fprintf(stderr, "\t[--libs]\n");
  fprintf(stderr, "\t[--cflags]\n");
  fprintf(stderr, "\t[--graphics]\n");
  fprintf(stderr, "Modes:\n");
  fprintf(stderr, "\tbasic\n");
  fprintf(stderr, "\tmotion\n");  
  fprintf(stderr, "\tnavigation\n");  
  
  exit(-1);
}

static void 
print_cflags(int mode __attribute__ ((unused)), int graphics)
{  
  printf("-I%s/include ", home);
  if (graphics)
    printf("-I/usr/lib/glib/include -I/usr/X11R6/include ");

  printf("\n");
}

static void 
print_libs(int mode __attribute__ ((unused)), int graphics)
{
  printf("-L%s/lib ", home);

  if (mode & 1) {
    if (!graphics) 
      printf("-lglobal -lipc -lparam_interface ");
    else 
      printf("-lglobal -lipc -lglobal_graphics -lparam_interface ");
  }

  if (mode & 2) {
    printf("-lgeometry -lbase_interface -lrobot_interface ");
  }

  if (mode & 4) {
    printf("-lmap_interface -lmap_io -lmap_util -llocalize_interface "
	   "-lnavigator_interface ");
    printf("-lz ");
  }

  if (graphics) {
    printf("-L/usr/lib -L/usr/X11R6/lib -lgtk -lgdk -rdynamic -lgmodule -lglib -ldl -lXi -lXext -lX11 -lm ");
    printf("-ljpeg ");
    if (mode & 4)
      printf("-lmap_graphics -lgdk_imlib ");
  }
  printf("\n");
}

int main(int argc, char *argv[]) 
{
  int cflags = 0;
  int libs = 0;
  int graphics = 0;
  int index;
  int mode = 0;
  char path_buffer[1024], buffer[1024];
  char *dir;

  if (argc == 1)
    usage(argc, argv);

  if (getenv("CARMEN_HOME")) 
    home = getenv("CARMEN_HOME");
  else 
    {
      sprintf(path_buffer, "/proc/%d/exe", getpid());
      memset(buffer, 0, 1024);
      readlink(path_buffer, buffer, 1024);
      home = buffer;

      dir = strrchr(buffer, '/');
      if (dir != NULL)
	{
	  dir[0] = '\0';
	  if (strrchr(buffer, '/'))
	    dir = strrchr(buffer, '/');
	  if (strlen(dir) >= 4 && strncmp(dir+1, "bin", 3) == 0)
	    dir[0] = '\0';
	  else if (dir - buffer >= 4 && strlen(dir) > 6 &&
		   strncmp(dir-4, "/src/global", 10) == 0)
	    dir[-4] = '\0';
	}
    }

  for (index = 1; index < argc; index++) 
    {
      if (strcmp(argv[index], "--version") == 0) 
	printf("%d.%d.%d\n", CARMEN_MAJOR_VERSION, CARMEN_MINOR_VERSION,
	       CARMEN_REVISION);
      else if (strcmp(argv[index], "--cflags") == 0) 
	cflags = 1;
      else if (strcmp(argv[index], "--libs") == 0) 
	libs = 1; 
      else if (strcmp(argv[index], "--graphics") == 0) 
	{
        graphics = 1;
	}
      else if (strcmp(argv[index], "basic") == 0) 
	mode |= 1;  
      else if (strcmp(argv[index], "motion") == 0) 
	mode |= 3;  
      else if (strcmp(argv[index], "navigation") == 0) 
	mode |= 7;      
      else
	usage(argc, argv);
    }

  if (cflags)
    print_cflags(mode, graphics);
  if (libs)
    print_libs(mode, graphics);

  return 0;
}
