/*********************************************************
 *
 * This source code is part of the Carnegie Mellon Robot
 * Navigation Toolkit (CARMEN)
 *
 * CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 * Roy, and Sebastian Thrun
 *
 * CARMEN is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option)
 * any later version.
 *
 * CARMEN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with CARMEN; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, 
 * Suite 330, Boston, MA  02111-1307 USA
 *
 ********************************************************/

/**********************************************
 * library of function for mapserver clients  *
 **********************************************/

#ifndef MAP_INTERFACE_H
#define MAP_INTERFACE_H

#include <carmen/map.h>
#include <carmen/map_messages.h>

#ifdef __cplusplus
extern "C" {
#endif


/* subscribe to map messages output by the map server */
void carmen_map_subscribe_gridmap_update_message(carmen_map_t *map, 
						 carmen_handler_t handler, 
						 carmen_subscribe_t 
						 subscribe_how);
/* request a map from the server */
int carmen_map_get_gridmap(carmen_map_p map);

/* request a map from the server */
int carmen_map_get_placelist(carmen_map_placelist_p placelist);

int carmen_map_get_offlimits(carmen_offlimits_p *offlimits, int *list_length);

int carmen_map_apply_offlimits_chunk_to_map(carmen_offlimits_p offlimits, 
					    int list_length, carmen_map_p map);

/* conversions between map_point_t's and other points */
/* conversions between map_point_t's and other points */
int carmen_map_to_world(carmen_map_point_p map_point, carmen_world_point_p world_point);
int carmen_world_to_map(carmen_world_point_p world_point, carmen_map_point_p map_point); 

int carmen_point_to_map(carmen_point_p point, carmen_map_point_p map_point, carmen_map_p map);

/* distance between two map_points in grid cells */
double carmen_distance_map(carmen_map_point_p p1, carmen_map_point_p p2);
double carmen_distance_world(carmen_world_point_p p1, carmen_world_point_p p2);

int carmen_map_cmp(carmen_map_point_p p1, carmen_map_point_p p2);
int carmen_world_cmp(carmen_world_point_p p1, carmen_world_point_p p2);

int carmen_map_to_trajectory(carmen_map_point_p carmen_map_point, carmen_traj_point_p traj_point);
int carmen_trajectory_to_map(carmen_traj_point_p traj_point, carmen_map_point_p carmen_map_point,
				  carmen_map_p map);

carmen_map_p carmen_map_copy(carmen_map_p map);
void carmen_map_destroy(carmen_map_p *map);

#ifdef __cplusplus
}
#endif

#endif
