/*********************************************************
 *
 * This source code is part of the Carnegie Mellon Robot
 * Navigation Toolkit (CARMEN)
 *
 * CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 * Roy, and Sebastian Thrun
 *
 * CARMEN is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option)
 * any later version.
 *
 * CARMEN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with CARMEN; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, 
 * Suite 330, Boston, MA  02111-1307 USA
 *
 ********************************************************/

#ifndef MAP_WRITE_H
#define MAP_WRITE_H

#ifndef NO_GRAPHICS
#include <gdk_imlib.h>
#endif

#include <carmen/map.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef NO_ZLIB
#include <zlib.h>
typedef gzFile carmen_map_file_p;
#define carmen_map_fopen gzopen
#define carmen_map_fgetc gzgetc
#define carmen_map_feof gzeof
#define carmen_map_fprintf gzprintf
#define carmen_map_fseek gzseek
#define carmen_map_fclose gzclose
#define carmen_map_fread(W, X, Y, Z) gzread((Z), (W), (X)) 
#define carmen_map_fwrite(W, X, Y, Z) gzwrite((Z), (W), (X)) 
#define carmen_map_fgets(X, Y, Z) gzgets((Z), (X), (Y))
#define carmen_map_fputc(X, Y) gzputc((Y), (X))
#else
typedef FILE * carmen_map_file_p;
#define carmen_map_fopen fopen
#define carmen_map_fgetc fgetc
#define carmen_map_feof feof
#define carmen_map_fprintf fprintf
#define carmen_map_fseek fseek
#define carmen_map_fclose fclose
#define carmen_map_fread(W, X, Y, Z) fread((W), (X), (Y), (Z)) 
#define carmen_map_fwrite(W, X, Y, Z) fwrite((W), (X), (Y), (Z)) 
#define carmen_map_fgets(X, Y, Z) fgets((X), (Y), (Z))
#define carmen_map_fputc(X, Y) fputc((X), (Y))
#endif

#define CARMEN_MAP_LABEL "CARMENMAPFILE"
#define CARMEN_MAP_VERSION "v020"

#define CARMEN_MAP_NO_CHUNK          (0 << 0)
#define CARMEN_MAP_GRIDMAP_CHUNK     (1 << 0)
#define CARMEN_MAP_OFFLIMITS_CHUNK   (1 << 1)
#define CARMEN_MAP_PLACES_CHUNK      (1 << 2)
#define CARMEN_MAP_EXPECTED_CHUNK    (1 << 3)
#define CARMEN_MAP_LASERSCANS_CHUNK  (1 << 4)
#define CARMEN_MAP_CREATOR_CHUNK     (1 << 5)


int carmen_map_write_all(carmen_map_file_p fp, float **prob,
			 int size_x, int size_y, double resolution,
			 char *comment_origin, char *comment_description,
			 char *creator_origin, char *creator_description,
			 carmen_place_p places, int num_places,
			 carmen_offlimits_p offlimits_list, int offlimits_num_items,
			 carmen_laser_scan_p scan_list, int num_scans);
int carmen_map_write_comment_chunk(carmen_map_file_p fp, int size_x, 
				   int size_y, double resolution, char *origin,
				   char *description);
int carmen_map_write_id(carmen_map_file_p fp);
int carmen_map_copy_comments(carmen_map_file_p fp_in, 
			     carmen_map_file_p fp_out);
int carmen_map_strip(carmen_map_file_p fp_in, carmen_map_file_p fp_out, 
		     int chunk);

int carmen_map_write_creator_chunk(carmen_map_file_p fp, char *origin, 
				   char *description);
int carmen_map_write_gridmap_chunk(carmen_map_file_p fp, float **prob, 
				   int size_x, int size_y, double resolution);
int carmen_map_write_places_chunk(carmen_map_file_p fp, carmen_place_p places, 
				  int num_places);
int carmen_map_write_offlimits_chunk(carmen_map_file_p fp, 
				     carmen_offlimits_p offlimits_list,
				     int num_items);
int carmen_map_write_laserscans_chunk(carmen_map_file_p fp, 
				      carmen_laser_scan_p scan_list,
				      int num_scans);
int carmen_map_chunk_exists(char *filename, int specific_chunk);
int carmen_map_advance_to_chunk(carmen_map_file_p fp, int specific_chunk);
int carmen_map_read_creator_chunk(char *filename, time_t *creation_time, 
				  char *username, char *origin, 
				  char *description);
int carmen_map_read_gridmap_config(char *filename, carmen_map_config_p config);
int carmen_map_read_gridmap_chunk(char *filename, carmen_map_p map);
int carmen_map_read_offlimits_chunk(char *filename, 
				    carmen_offlimits_p *offlimits_list,
				    int *list_length);
int carmen_map_read_offlimits_chunk_into_map(char *filename, carmen_map_p map);
int carmen_map_read_places_chunk(char *filename, 
				 carmen_map_placelist_p places);
int carmen_map_read_laserscans_chunk(char *filename, 
				     carmen_laser_scan_p *scan_list, 
				     int *num_scans);
int carmen_map_file(char *filename);
int carmen_map_initialize_ipc(void);
void carmen_map_set_filename(char *new_filename);
void carmen_map_publish_update(void);


#ifndef NO_GRAPHICS
carmen_map_p carmen_map_image_to_map( GdkImlibImage* im, double resolution);
carmen_map_p carmen_map_imagefile_to_map(char *filename, double resolution);
#endif

#ifdef __cplusplus
}
#endif

#endif
