/*********************************************************
 *
 * This source code is part of the Carnegie Mellon Robot
 * Navigation Toolkit (CARMEN)
 *
 * CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 * Roy, and Sebastian Thrun
 *
 * CARMEN is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option)
 * any later version.
 *
 * CARMEN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with CARMEN; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, 
 * Suite 330, Boston, MA  02111-1307 USA
 *
 ********************************************************/

#include <carmen/carmen.h>
#include "map_io.h"

static void write_to_ppm(carmen_map_p map)
{
  int x, y;
  char c;

  printf("P6\n%d %d\n255\n", map->config.x_size, map->config.y_size);
  for(y = map->config.y_size - 1; y >= 0; y--)
    for(x = 0; x < map->config.x_size; x++) 
      if(map->map[x][y] == -1) {
	fputc(0, stdout); fputc(0, stdout); fputc(255, stdout);
      }
      else if(map->map[x][y] >= 2.0) {
	c = 255 - 255 * (map->map[x][y] - 2.0);
	fputc(255, stdout); fputc(c, stdout); fputc(c, stdout);
      }
      else {
	c = 255 - 255 * map->map[x][y];
	fputc(c, stdout); fputc(c, stdout); fputc(c, stdout);
      }
}

int main(int argc, char **argv)
{
  carmen_map_t map;

  if(argc < 2) {
    fprintf(stderr, "Error: wrong number of parameters.\n");
    fprintf(stderr, "Usage: %s mapfilename\n", argv[0]);
    return -1;
  }
  if(carmen_map_read_gridmap_chunk(argv[1], &map) < 0) {
    fprintf(stderr, "Error: This file did not contain a gridmap.\n");
    return -1;
  }
  carmen_map_read_offlimits_chunk_into_map(argv[1], &map);
  write_to_ppm(&map);
  return 0;
}
