/*********************************************************
 *
 * This source code is part of the Carnegie Mellon Robot
 * Navigation Toolkit (CARMEN)
 *
 * CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 * Roy, and Sebastian Thrun
 *
 * CARMEN is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option)
 * any later version.
 *
 * CARMEN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with CARMEN; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, 
 * Suite 330, Boston, MA  02111-1307 USA
 *
 ********************************************************/

#include <carmen/carmen.h>
#include "map_io.h"

int carmen_map_advance_to_chunk(carmen_map_file_p fp, int specific_chunk);
int carmen_map_write_offlimits_chunk(carmen_map_file_p fp, 
				     carmen_offlimits_p offlimits_list,
				     int num_items);

int
main(int argc, char **argv)
{
  char *filename;
  carmen_map_file_p in_fp, out_fp;
  int ret_val;
  carmen_map_t map;
  int x_offset, y_offset;
  char backup_filename[1024];
  char new_filename[1024];
  carmen_offlimits_list_t offlimits_list;

  if (argc != 2)
    carmen_die("Usage: minimize <mapfile>\n");

  filename = argv[1];

  in_fp = carmen_map_fopen(filename, "r");
  if (in_fp == NULL)
    carmen_die_syserror("Couldn't open %s for reading", filename);

  sprintf(new_filename, "%s.new", filename);
  out_fp = carmen_map_fopen(new_filename, "w");
  if (out_fp == NULL)
    carmen_die_syserror("Couldn't open %s for writing", new_filename);

  if (carmen_map_strip(in_fp, out_fp, CARMEN_MAP_GRIDMAP_CHUNK | 
		       CARMEN_MAP_OFFLIMITS_CHUNK) < 0)
    {
      unlink(new_filename);
      carmen_die_syserror("Couldn't strip map to %s", new_filename);
    }

  ret_val = carmen_map_chunk_exists(filename, CARMEN_MAP_GRIDMAP_CHUNK);

  if (ret_val < 0)    
    {
      unlink(new_filename);
      carmen_die_syserror("Couldn't check existence of GRIDMAP_CHUNK in %s",
			  filename);
    }

  if (ret_val ==  0)
    {
      unlink(new_filename);
      carmen_die("There is no gridmap in %s\n", filename);
    }

  if (carmen_map_read_gridmap_chunk(filename, &map) < 0)
    {
      unlink(new_filename);
      carmen_die_syserror("Couldn't read GRIDMAP_CHUNK from %s", filename);
    }

  carmen_warn("Map size was %d x %d, ",map.config.x_size, map.config.y_size);

  carmen_map_minimize_gridmap(&map, &x_offset, &y_offset);

  carmen_warn("is now %d x %d (offset %d, %d)\n", map.config.x_size, 
	      map.config.y_size, x_offset, y_offset);

  carmen_map_write_gridmap_chunk(out_fp, map.map, map.config.x_size, 
				 map.config.y_size, map.config.resolution);

  carmen_map_read_offlimits_chunk(filename, &(offlimits_list.offlimits), 
				  &(offlimits_list.list_length));

  carmen_map_move_offlimits_chunk(&offlimits_list, x_offset, y_offset);

  carmen_map_write_offlimits_chunk(out_fp, offlimits_list.offlimits,
				   offlimits_list.list_length);

  carmen_map_fclose(in_fp);
  carmen_map_fclose(out_fp);

  sprintf(backup_filename, "%s.bak", filename);
  if (rename(filename, backup_filename) < 0)
    {
      unlink(new_filename);
      carmen_die("Couldn't rename %s to %s : %s\n", filename, backup_filename, 
							 strerror(errno));
    }

  if (rename(new_filename, filename) < 0)
    carmen_die("Couldn't rename %s to %s : %s\n", new_filename, filename, 
						 strerror(errno));

  return 0;
}
