/*********************************************************
 *
 * This source code is part of the Carnegie Mellon Robot
 * Navigation Toolkit (CARMEN)
 *
 * CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 * Roy, and Sebastian Thrun
 *
 * CARMEN is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option)
 * any later version.
 *
 * CARMEN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with CARMEN; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, 
 * Suite 330, Boston, MA  02111-1307 USA
 *
 ********************************************************/

#include <carmen/carmen.h>
#include "map_io.h"

void 
rotate_places_chunk(carmen_place_p places, int num_places, carmen_map_p map, 
		    int rotation)
{
  int index;
  int width, height;
  double tmp;

  width = map->config.x_size;
  height = map->config.y_size;

  for (index = 0; index < num_places; index++)
    {
      if (rotation == 1)
	{
	  tmp = places[index].x;
	  places[index].x = height-places[index].y;
	  places[index].y = tmp;
	}
      else if (rotation == 2)
	{
	  places[index].x = width - places[index].x;
	  places[index].y = height - places[index].y;
	}
      else if (rotation == 3)
	{
	  tmp = places[index].x;
	  places[index].x = places[index].y;
	  places[index].y = width-tmp;
	}
    }
}

void
rotate_gridmap(carmen_map_p map, int rotation)
{
  int index, x = 0, y = 0;
  int height, width;
  float *new_map, *new_map_ptr;

  new_map = (float *)calloc(map->config.x_size*map->config.y_size, 
			    sizeof(float));
  carmen_test_alloc(new_map);

  new_map_ptr = new_map;
  width = map->config.x_size;
  height = map->config.y_size;
  for (index = 0; index < width*height; index++)
    {      
      if (rotation == 0) 
	{
	  x = index / height;
	  y = index % height;
	}	
      else if (rotation == 1)
	{
	  x = width - (index % width) - 1;
	  y = index / width;
	}
      else if (rotation == 2)
	{
	  x = width - (index / height) - 1;
	  y = height - (index % height) - 1;
	}	
      else if (rotation == 3)
	{
	  x = index % width;
	  y = width - index / width - 1;
	}
      *(new_map_ptr++) = map->map[x][y];
    }

  free(map->complete_map);
  map->complete_map = new_map;

  free(map->map);
  
  if (rotation == 1 || rotation == 3) {
    height = map->config.x_size;
    width = map->config.y_size;
    map->config.x_size = width;
    map->config.y_size = height;
  }

  map->map = (float **)calloc(map->config.x_size, sizeof(float *));
  carmen_test_alloc(map->map);
  
  for (index = 0; index < map->config.x_size; index++) 
    map->map[index] = map->complete_map+index*map->config.y_size;
}

int
main(int argc, char **argv)
{
  char *in_filename;
  carmen_map_file_p in_fp, out_fp;
  int ret_val;
  carmen_map_t map;
  char *out_filename;
  int rotation = 0;
  double remain;
  int degrees_angle;

  if (argc != 4)
    carmen_die("Usage: rotate <rotation in degs> <input file> "
	       "<output file>\n");

  degrees_angle = (int)(atof(argv[1]) / 90);
  remain = fabs(degrees_angle*90 - atof(argv[1]));
  if (carmen_radians_to_degrees(remain) > 2)
    carmen_die("Rotations only supported in increments of 90 degrees.\n");
  else
    rotation = (int)atof(argv[1]) / 90;

  in_filename = argv[2];
  out_filename = argv[3];

  carmen_warn("Rotating by %d degrees\n", rotation*90);

  ret_val = carmen_map_chunk_exists(in_filename, CARMEN_MAP_GRIDMAP_CHUNK);
	
  if (ret_val < 0)    
      carmen_die_syserror("Couldn't check existence of GRIDMAP_CHUNK in %s", 
			  in_filename);

  if (carmen_map_read_gridmap_chunk(in_filename, &map) < 0)
      carmen_die_syserror("Couldn't read GRIDMAP_CHUNK from %s", in_filename);

  rotate_gridmap(&map, rotation);

  in_fp = carmen_map_fopen(in_filename, "r");
  if (in_fp == NULL)
    carmen_die_syserror("Couldn't open %s for reading", in_filename);

  out_fp = carmen_map_fopen(out_filename, "w");
  if (out_fp == NULL)
    carmen_die_syserror("Couldn't open %s for writing", out_filename);

  ret_val = carmen_map_strip(in_fp, out_fp, CARMEN_MAP_GRIDMAP_CHUNK);

  ret_val = carmen_map_write_gridmap_chunk(out_fp, map.map, map.config.x_size, 
					   map.config.y_size, 
					   map.config.resolution);

  carmen_map_fclose(in_fp);
  carmen_map_fclose(out_fp);

  return 0;
}
