/*********************************************************
 *
 * This source code is part of the Carnegie Mellon Robot
 * Navigation Toolkit (CARMEN)
 *
 * CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 * Roy, and Sebastian Thrun
 *
 * CARMEN is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option)
 * any later version.
 *
 * CARMEN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with CARMEN; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, 
 * Suite 330, Boston, MA  02111-1307 USA
 *
 ********************************************************/

#include <carmen/carmen.h>
#include "map_io.h"

int main(int argc, char **argv)
{
  char *filename, cmd[100];
  carmen_map_file_p fp_in, fp_out;
  int chunks, index;

  if (argc < 3) 
    {
      fprintf(stderr, "Error: wrong number of arguments.\n");
      fprintf(stderr, "Usage: %s mapfilename chunkname chunkname2 ... \n", 
	      argv[0]);
      fprintf(stderr, "chunk name can be one of \"laserscans\", "
	      "\"places\", \n");
      fprintf(stderr, "      \"gridmap\", \"offlimits\", \"expected\"\n");
      return -1;
    }

  filename = argv[1];
  chunks = 0;
  for (index = 2; index < argc; index++)
    {
      if (carmen_strcasecmp(argv[index], "laserscans") == 0)
	chunks |= CARMEN_MAP_LASERSCANS_CHUNK;
      if (carmen_strcasecmp(argv[index], "gridmap") == 0)
	chunks |= CARMEN_MAP_GRIDMAP_CHUNK;
      if (carmen_strcasecmp(argv[index], "offlimits") == 0)
	chunks |= CARMEN_MAP_OFFLIMITS_CHUNK;
      if (carmen_strcasecmp(argv[index], "expected") == 0)
	chunks |= CARMEN_MAP_EXPECTED_CHUNK;
      if (carmen_strcasecmp(argv[index], "places") == 0)
	chunks |= CARMEN_MAP_PLACES_CHUNK;
    }

  fp_in = carmen_map_fopen(filename, "r");
  if(fp_in == NULL) 
    carmen_die_syserror("Could not open map file for reading");

  fp_out = carmen_map_fopen("/tmp/stripped.map", "w");
  if(fp_out == NULL) 
    carmen_die_syserror("Could not open temporary file for writing");
  
  if(carmen_map_strip(fp_in, fp_out, chunks) < 0) 
    carmen_die_syserror("Error: could not strip file");
 
  carmen_map_fclose(fp_in);
  carmen_map_fclose(fp_out);
  sprintf(cmd, "mv -f /tmp/stripped.map %s", filename);
  system(cmd);
  return 0;
}
