/*********************************************************
 *
 * This source code is part of the Carnegie Mellon Robot
 * Navigation Toolkit (CARMEN)
 *
 * CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 * Roy, and Sebastian Thrun
 *
 * CARMEN is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option)
 * any later version.
 *
 * CARMEN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with CARMEN; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, 
 * Suite 330, Boston, MA  02111-1307 USA
 *
 ********************************************************/

#include <carmen/carmen.h>
#include "pioneer_main.h"

static int shutdown_in_progress = 0;

void 
shutdown_pioneer(int signo) 
{
  if (shutdown_in_progress)
    return;
  shutdown_in_progress = 1;

  carmen_pioneer_shutdown(signo);
  exit(0);
}

int 
main(int argc, char **argv) 
{
  carmen_initialize_ipc(argv[0]);
  carmen_param_check_version(argv[0]);

  signal(SIGINT, shutdown_pioneer);
  signal(SIGTERM, shutdown_pioneer);

  if (carmen_pioneer_start(argc, argv) < 0)
    exit(-1);

  while(1) {
    fprintf(stderr, ".");
    sleep_ipc(0.01);
    carmen_pioneer_run();
  }
  return 0;
}
